// TT-Tachometer 12. for the v3.0 board with external clock feeding Timer1, works
// tests display and shuts off display if turntable not spinning.
//Global Variables
#define TACH_PIN 2  
#define CLOCK_PIN 5
//set volatile for ISR variable
volatile float timePast = 0;
float previousTime = 0;
float rpm = 0;
unsigned long number1 = 0; // for float to int conversion
volatile unsigned long num_overflows = 0;
unsigned long count = 0;

const int numeral[10] = { //ABCDEFG,dp segments to light for number
  B11111100, //0
  B01100000, //1
  B11011010, //2
  B11110010, //3
  B01100110, //4
  B10110110, //5
  B00111110, //6
  B11100000, //7
  B11111110, //8
  B11100110, //9
};

//pins for decimal point and each segment, dp, G, F, E, D, C, B, A
const int segmentPins[] = {14, 12, 11, 10, 9, 8, 7, 6}; 
const int numberofDigits = 5; //number of seven segment LED display parts
const int digitPins[numberofDigits] = {19, 18, 17, 16, 15}; //digits 1, 2, 3, 4, 5

void setup() {
  TCCR1B = 0b00000111; //external clock source
  TIMSK1 = 0b00000001; //enable Timer 1 overflow interrupt.
  TIMSK0 = 0b00000000; //disable Timer0 interrupts.
  TIMSK2 = 0b00000000; //disable Timer2 interrupts.
  TCCR1A = 0; //set WGM21 and WGM20 to 0
  pinMode(TACH_PIN, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(TACH_PIN), isr1, FALLING);
  pinMode(CLOCK_PIN, INPUT); 
  pinMode(6, OUTPUT); // set pins to output mode to light LED segments
  pinMode(7, OUTPUT);
  pinMode(8, OUTPUT);
  pinMode(9, OUTPUT);
  pinMode(10, OUTPUT);
  pinMode(11, OUTPUT);
  pinMode(12, OUTPUT);
  pinMode(14, OUTPUT); // Decimal Point
  pinMode(15, OUTPUT); // set pins to output mode for LED CC control on
  pinMode(16, OUTPUT);
  pinMode(17, OUTPUT);
  pinMode(18, OUTPUT);
  pinMode(19, OUTPUT);
  digitalWrite(14, HIGH); //Turn on decimal point
  for(unsigned long i=1; i<640; i++)
    showNumber(88888);
} //end setup

// Interrupt Service Routine 1. Saves the time from the clock var.
void isr1(){
  timePast = get_count();
}

// Interrupt Service Routine 2. count the number of overflows.
// one overflow counts as 65536 clock pulses.
ISR(TIMER1_OVF_vect){
  num_overflows++;
}

volatile unsigned long get_count(){
  uint8_t SREG_Save = SREG; //Save the AVR Status Register
  noInterrupts();
  unsigned long tcnt1_Save = TCNT1; //save counter of Timer1
  boolean OVFFLAG = bitRead(TIFR1, 0); //Timer1 Overflow Flag
  if(OVFFLAG){
    tcnt1_Save = TCNT1; //re-read timer1 in case changed
    num_overflows++;
    TIFR1 |= 0b00000001; //reset Timer1 overflow flag
  }  
  count = ((num_overflows*65536) + tcnt1_Save);
  num_overflows = 0;
  TCNT1 = 0;
  SREG = SREG_Save; //enable interrupts as previously set
  return count;
}

void loop() {
  if (num_overflows > 5){  // turntable not turning
    digitalWrite(15, HIGH); //Turn off all digits
    digitalWrite(16, HIGH); 
    digitalWrite(17, HIGH);
    digitalWrite(18, HIGH);
    digitalWrite(19, HIGH); 
  }
  else{ 
    if((timePast - previousTime) != 0){
      rpm = 6000000/timePast; //float rpm calc  
      number1 = rpm*1000; // convert float to 5 digit whole number
    }
    previousTime = timePast; //update previous time each pass, remove calc unless change
    showNumber(number1); // show five digit on LEDs
   } 
} // end loop

void showNumber (unsigned long number){ //number should be 5 digits
  if (number == 0)  //if number is zero only process once
    showDigit(0, numberofDigits - 1); //display 0 in the rightmost digit
  else{
    for (int digit = numberofDigits - 1; digit >= 0; digit--){ //starts at right most digit
      if (number > 0){ //Don't show leading zeros
        showDigit(number % 10, digit); // Send next digit and digit position
        number = number / 10;  // Remove the digit so only ones to show left
      }//end if
    }//end for
  }//end else
}//end showNumber()

//Displays given number on a 7-segment display at the given digit position
void showDigit (int number, int digitx){
  digitalWrite(digitPins[digitx], LOW); // turn digit on
  for (int segment = 1; segment < 8; segment++){ // walk thru the segments of the LED by number
    boolean isBitSet = bitRead(numeral[number], segment); // check if segment bit is on
    digitalWrite(segmentPins[segment], isBitSet); //turn segment on or off
  }//end for loop

  delay(3); //controls illumination, time lit before off
  digitalWrite(digitPins[digitx], HIGH); // Turn digit off
}//end showDigit()
